const config = require('config')
const { get } = require('lodash')

const unsubscribeSlug = config.get('unsubscribe.url')
const inviteReviewerPath = config.get('invite-reviewer.url')

const {
  Email,
  services,
  Fragment,
} = require('pubsweet-component-helper-service')

const { getEmailCopy } = require('./emailCopy')

module.exports = {
  async sendInvitations({
    resend,
    baseUrl,
    fragment,
    UserModel,
    timestamp,
    collection,
    invitation,
    invitedUser,
  }) {
    const fragmentHelper = new Fragment({ fragment })
    const { title, abstract } = await fragmentHelper.getFragmentData({
      handlingEditor: collection.handlingEditor,
    })
    const {
      activeAuthors: authors,
      submittingAuthor,
    } = await fragmentHelper.getAuthorData({
      UserModel,
    })

    const subjectBaseText = `${collection.customId}: Review`

    let queryParams = {
      invitationId: invitation.id,
      agree: true,
    }

    const detailsPath = `/projects/${collection.id}/versions/${
      fragment.id
    }/details`

    const declineLink = services.createUrl(baseUrl, inviteReviewerPath, {
      ...queryParams,
      agree: false,
      fragmentId: fragment.id,
      collectionId: collection.id,
      invitationToken: invitedUser.invitationToken,
    })

    let agreeLink = services.createUrl(baseUrl, detailsPath, queryParams)

    if (!invitedUser.isConfirmed) {
      queryParams = {
        ...queryParams,
        email: invitedUser.email,
        token: invitedUser.passwordResetToken,
        collectionId: collection.id,
        fragmentId: fragment.id,
        agree: true,
      }
      agreeLink = services.createUrl(baseUrl, inviteReviewerPath, queryParams)
    }

    const authorsList = authors.map(
      author => `${author.firstName} ${author.lastName}`,
    )

    const email = new Email({
      type: 'user',
      toUser: {
        email: invitedUser.email,
        name: `${invitedUser.lastName}`,
      },
      content: {
        title,
        abstract,
        agreeLink,
        declineLink,
        authorsList: authorsList.join(', '),
        subject: `${subjectBaseText} invitation`,
        detailsLink: services.createUrl(baseUrl, detailsPath),
        signatureName: get(collection, 'handlingEditor.name', 'Hindawi'),
        unsubscribeLink: services.createUrl(baseUrl, unsubscribeSlug, {
          id: invitedUser.id,
        }),
      },
    })

    sendInvitedUserEmail({
      email,
      title,
      resend,
      timestamp,
      authorName: `${submittingAuthor.firstName} ${submittingAuthor.lastName}`,
    })
  },
}

const sendInvitedUserEmail = async ({
  email,
  resend,
  title,
  authorName,
  timestamp,
}) => {
  let daysExpected = 14
  let emailType = 'reviewer-invitation'
  let titleText = `A manuscript titled "${title}" by ${authorName} et al.`

  if (resend) {
    emailType = 'reviewer-resend-invitation'
    daysExpected = 0
    email.content.subject = `${email.content.subject} reminder`
    titleText = `the manuscript titled "${title}" by ${authorName}`
  }

  const { html, text } = email.getBody({
    isReviewerInvitation: true,
    body: getEmailCopy({
      emailType,
      titleText,
      expectedDate: services.getExpectedDate({ timestamp, daysExpected }),
    }),
  })

  email.sendEmail({ html, text })
}
