const config = require('config')
const { get } = require('lodash')

const unsubscribeSlug = config.get('unsubscribe.url')
const fromEmail = config.get('journal.staffEmail')
const journalName = config.get('journal.name')

const {
  User,
  Email,
  services,
  Fragment,
} = require('pubsweet-component-helper-service')

const { getEmailCopy } = require('./emailCopy')

module.exports = {
  async sendNotifications({
    baseUrl,
    fragment,
    reviewer,
    UserModel,
    emailType,
    collection,
    isCanceled = false,
  }) {
    const fragmentHelper = new Fragment({ fragment })
    const { title } = await fragmentHelper.getFragmentData({
      handlingEditor: collection.handlingEditor,
    })
    const { submittingAuthor } = await fragmentHelper.getAuthorData({
      UserModel,
    })

    const titleText = `the manuscript titled "${title}" by ${
      submittingAuthor.firstName
    } ${submittingAuthor.lastName}`

    const handlingEditor = get(collection, 'handlingEditor')
    const userHelper = new User({ UserModel })
    const eicName = await userHelper.getEiCName()

    const email = new Email({
      fromEmail,
      type: 'user',
      content: {
        ctaText: 'MANUSCRIPT DETAILS',
        signatureJournal: journalName,
        signatureName: handlingEditor.name,
        ctaLink: services.createUrl(
          baseUrl,
          `/projects/${collection.id}/versions/${fragment.id}/details`,
        ),
      },
    })

    if (emailType !== 'reviewer-declined') {
      sendReviewerEmail({
        email,
        baseUrl,
        reviewer,
        titleText,
        isCanceled,
        customId: collection.customId,
      })
    }

    if (['reviewer-accepted', 'reviewer-declined'].includes(emailType)) {
      sendHandlingEditorEmail({
        email,
        eicName,
        titleText,
        emailType,
        handlingEditor,
        customId: collection.customId,
        targetUserName: `${reviewer.lastName}`,
      })
    }
  },
}

const sendHandlingEditorEmail = ({
  email,
  eicName,
  baseUrl,
  customId,
  titleText,
  emailType,
  handlingEditor,
  targetUserName,
}) => {
  email.toUser = {
    email: handlingEditor.email,
    name: handlingEditor.name,
  }

  email.content.subject =
    emailType === 'reviewer-accepted'
      ? `${customId}: A reviewer has agreed`
      : `${customId}: A reviewer has declined`
  email.content.unsubscribeLink = services.createUrl(baseUrl, unsubscribeSlug, {
    id: handlingEditor.id,
  })
  email.content.signatureName = eicName
  email.content.signatureJournal = ''

  const { html, text } = email.getBody({
    body: getEmailCopy({
      emailType,
      titleText,
      expectedDate: services.getExpectedDate({
        timestamp: Date.now(),
        daysExpected: 14,
      }),
      targetUserName,
    }),
  })
  email.sendEmail({ html, text })
}

const sendReviewerEmail = async ({
  email,
  baseUrl,
  reviewer,
  customId,
  titleText,
  isCanceled,
}) => {
  email.content.subject = isCanceled
    ? `${customId}: Review no longer required`
    : `${customId}: Thank you for agreeing to review`

  const emailType = isCanceled
    ? 'reviewer-cancel-invitation'
    : 'reviewer-thank-you'

  email.toUser = {
    email: reviewer.email,
    name: `${reviewer.lastName}`,
  }

  email.content.unsubscribeLink = services.createUrl(baseUrl, unsubscribeSlug, {
    id: reviewer.id,
  })

  const { html, text } = email.getBody({
    body: getEmailCopy({
      emailType,
      titleText,
    }),
  })

  email.sendEmail({ html, text })
}
